#!/bin/sh
#
# 	(c) FRISK Software International
#	author: Svavar Ingi Hermannsson 2002
#
# 	http://www.f-prot.com/
#
# If you want to run this script in a cronjob, then you
# should give the cron parameter, here's an example of
# a crontab entry which runs check-updates twice a day: 04:27 and
# 16:27.
#	(with this configuration, you will get e-mails whenever
#	signature files are updated or when an error occours)
#
# 27 4,16 * * *	/usr/local/f-prot/check-updates -cron
#
#
#	(If you only want to receive e-mail only when an error,
#	occours, then you want to include the -quiet parameter)
#	
# 27 4,16 * * * /usr/local/f-prot/check-updates -cron -quiet
#
# (consult your crontab manual page, this format may differ between
# versions)

# Make sure we use BASH.
if [ -z "$BASH_VERSION" ]; then
        BASH=`which bash`
        if [ $? -ne 0 ]; then
                echo "Error: You need Bash installed to run this script."
                exit 1
        fi
        BASH=`echo $BASH | awk '{print $1}'`
        exec $BASH $0 ${@+"$@"}
fi

# Here we declare our variables.
PARAMETERS=$@
CRON=0
QUIET=0
TEMPDIR="/var/tmp/f-prot/"
DOWNLOADSIGN=0
DEFDIR="/usr/local/f-prot/"
SERVER=""
FALLBACKSERVER="http://updates.f-prot.com/files/"
FPROTDIR=$DEFDIR
TMPFILE="tmp-web"
DOWNLOADFILE=""
DOWNLOADSERVER=""
FILETOCHECK=""
FILECHECKSUM=""
FILETODOWNLOAD=""
REMOTEFILECHECKSUM=""
HTTPRETURN=10
CMDRESULT=""
UPDATED=0
SIGNINSTALLED=0
NEWFILE=""

# A function to check the parameters we are given.
parseParameters ()
{

	# Here we check if we're running in cron mode
	until [ -z $1 ]
	do
		case $1 in
		"cron" | "-cron" | "--cron" | "CRON" | "-CRON" | "--CRON")
			CRON=1
		;;
	
		"quiet" | "-quiet" | "--quiet" | "QUIET" | "-QUIET" | "--QUIET")
			QUIET=1
		;;

		*)
			echo "Invalid option..."
			exit -1
		esac

		shift
	done

	return 0
}


# A function which checks if the temporary directory we are using
# exists, and if it doesn't it tries to create it.
checkIfTempDirExists ()
{
	# Here we make sure that our temporary directory exists,
	# and if it doesn't we will make one.
	if [ -e ${TEMPDIR%/} ]
	then
		if [ -f ${TEMPDIR%/} ]
		then
			# if the location tempdir is pointing to, is a file and
			# not a directory then we exit. 
			printHeader
			echo "${TEMPDIR} is a file, not a directory. You can"
			echo "either change the location of the TEMPDIR in the"
			echo "script, or you can remove this file and let this"
			echo "script create a new directory."
			echo ""
			exit -12

		elif [ -h ${TEMPDIR%/} ]
		then
			echo "Error: There exists a symbolic link with the same name as the"
			echo "temporary directory you've declared: ${TEMPDIR}"
			echo "Please remove this link and restart the script: ${0}"
			echo "Exiting..."
			exit -13

		elif [ ! -O ${TEMPDIR%/} ]
		then
			echo "Error: You are not the owner of the temporary directory"
			echo "which this script is using: ${TEMPDIR}. "
			echo "Please change the file permissions and try again"
			echo "Exiting..."
			exit -13
		fi
		
	else
		# We make the temporary directory. 
		mkdir ${TEMPDIR}
		CMDRESULT=$?
		if [ $CMDRESULT -ne 0 ]
		then
			echo "Unable to make temporary directory: ${TEMPDIR}."
			echo "Make sure the TEMPDIR variable is correct and "
			echo "that all file permissions are correct."
			echo ""
			echo "Fatal error: exiting..."
			exit -13
		fi
	fi

	return 0
}

# A function which makes sure only the owner of the script has
# write access to the temporary directory to prevent a race-condition
# in the case of other users.
changePermissionsOfTempDir ()
{
	# we want to make sure the file permissions are correct to
	# prevent race conditions... 
	chmod 0700 ${TEMPDIR}
	CMDRESULT=$?
	if [ $CMDRESULT -ne 0 ]
	then
		echo "Unable to change access permissions for: ${TEMPDIR}."
		echo "Make sure you have ownership over this directory."
		echo ""
		echo "Fatal error: exiting..."
		exit -14
	fi
	
	return 0
}

# A function which removes everything from the temporary directory.
cleanTempDir ()
{
	# Here we clean everything within the tempdir
	(cd ${TEMPDIR} && rm -f ./*)
	CMDRESULT=$?
	if [ $CMDRESULT -ne 0 ]
	then
		echo "Unable to clean temporary directory: ${TEMPDIR}."
		echo "Make sure you have ownership over this directory."
		echo ""
		echo "Error: exiting..."
		exit -15
	fi
	
	return 0
}

# A function which makes sure that we have all the external programs
# we need.
checkForExternalPrograms ()
{
	for program in grep head wget unzip
	do
		which $program > /dev/null 2>&1
		CMDRESULT=$?
		if [ $CMDRESULT -eq 127 ]
		then
			echo "${program} is not installed on your operating system,"
			echo "or it's location is not in your path variable."
			echo "Please install this software or fix your path"
			echo "variable, and try again"
			echo ""
			exit 127
		fi
	done 
}

# A function which downloads update information (checksums and etc.) from
# the update server.
downloadUpdateInfo ()
{
	wget --output-document=${TEMPDIR}${TMPFILE} --tries=3 'http://updates.f-prot.com/cgi-bin/check-updates?protocol=1&run_as=check_updates' > /dev/null 2>&1
	CMDRESULT=$?
	if [ $CMDRESULT -ne 0 ]
	then
		if [ $CMDRESULT -eq 127 ]
		then
			echo "Command wget not found, you need to install the latest"
			echo "version of wget to be able to use this script."
			echo "Fatal error: exiting..."
	
			exit -127
		else 
			echo "Error checking for updates, unable to connect to remote"
			echo "website, or wrong permissions on ${TEMPDIR}. "
			echo "Fatal error, exiting..."
			exit -1
		fi
	fi
}

# A function which gets the return value form the update server.
getHTTPReturnValue ()
{
	HTTPRETURN="`head -1 ${TEMPDIR}${TMPFILE}`"
	HTTPRETURN=${HTTPRETURN:0:1}
}

printHeader ()
{
	echo "***************************************"
	echo "* F-Prot signature file update script *"
	echo "***************************************"
	echo ""
}

printUpdateCompleted ()
{
	echo ""
	echo "**********************************"
	echo "* Update completed successfully. *"
	echo "**********************************"
	echo ""
}

downloadCompleted ()
{
	echo "Download completed."
	
	return 0
}

fileInstallCompleted ()
{
	if [ "${FILETOCHECK:0:4}" = "SIGN" ]
	then
		if [ $SIGNINSTALLED -eq 0 ]
		then
			SIGNINSTALLED=1
			echo "\"${NEWFILE}\" signatures have been installed."
		fi
	else
		echo "\"${NEWFILE}\" signatures have been installed."
	fi
	echo ""
}

# If we got 3 as the HTTP return value it means we sent an 
# invalid parameter (this should not happen).
invalidParameter ()
{
	echo "Invalid parameters used in http URL."
	echo "Fatal error: exiting..."
	exit 3
}

invalidProtocol ()
{
	echo "Invalid protocol in HTTP URL"
	echo "Make sure that the protocol=1 in the URL."
	echo "Fatal error: exiting..."

	exit 4
}

remoteServerError ()
{
	echo "Server error on remote machine."
	echo "Fatal error: exiting..."
	exit 5
}

unknownError ()
{
	echo "Unknown error. Exiting..."
	exit -1
}

unzipError ()
{
	echo "Fatal error, error while unzipping file. Please"
	echo "check the file permissions and try again."
	echo ""
	echo "Fatal error: exiting..."
	exit 9
}
copyError ()
{
	echo "Unable to copy file to ${FPROTDIR}."
	echo "Possible cause: wrong file permissions."
	echo ""
	echo "Fatal error: exiting..."
	echo ""
	exit 11
}

downloadError ()
{
	# FILETODOWNLOAD has been set globally by downloadFileFromServer()
	echo "Error while downloading file: ${FILETODOWNLOAD}."
	echo "Fatal error, please make sure your network connection is"
	echo "up and running."
	echo ""
	echo "Fatal error: exiting..."
	exit 19
}

checksumError ()
{
	echo "The files are not identical and therefore, they will"
	echo "not be installed. This could be caused by downloading"
	echo "from a mirror which hasn't synced yet with the main "
	echo "server. (This should not happen)"
	echo ""
	echo "Remote checksum: ${REMOTEFILECHECKSUM} "
	echo "Local checksum: ${FILECHECKSUM} "
	echo ""

	exit 25
}

checkHTTPReturnValue ()
{
	if [ $HTTPRETURN -ne "2" ]
	then
		if [ $HTTPRETURN -eq "3" ]
		then
			invalidParameter

		elif [ $HTTPRETURN -eq "4" ]
		then 
			invalidProtocol

		elif [ $HTTPRETURN -eq "5" ]
		then	
			remoteServerError
		else
			unknownError
		fi		
	fi

	return 0
}

downloadFromServer ()
{
	DOWNLOADFILE=${FILETOCHECK:0:4}
	if [ "$DOWNLOADFILE" = "SIGN" ] 
	then
		# If DOWNLOADSIGN = 0, it means we haven't downloaded 
		# anything yet, but if it is equal to one, we have already
		# downloaded the def file for SIGN.
		if [ $DOWNLOADSIGN -eq 0 ]
		then
			newSignatureFile
			DOWNLOADSIGN=1
			cd ${TEMPDIR}
			downloadFileFromServer $SERVER fp-def.zip

			if [ $QUIET -eq 0 ]
			then
				downloadCompleted
			fi

			unzip -o fp-def.zip > /dev/null 2>&1
			CMDRESULT=$?
			if [ $CMDRESULT -ne 0 ]
			then
					unzipError
			fi
		fi
	else
		# we know there are only 2 .zip files
		# fp-def.zip and macrdef2.zip
		newSignatureFile
		cd ${TEMPDIR}
		downloadFileFromServer $SERVER macrdef2.zip
		if [ $QUIET -eq 0 ]
		then
			downloadCompleted
		fi

                unzip -o macrdef2.zip > /dev/null 2>&1
		CMDRESULT=$?
		if [ $CMDRESULT -ne 0 ]
		then
			unzipError
		fi
	fi
	
	return 0
}

downloadFileFromServer ()
{
	DOWNLOADSERVER=$1
	FILETODOWNLOAD=$2
	wget --passive-ftp --tries=3 ${DOWNLOADSERVER}${FILETODOWNLOAD} > /dev/null 2>&1
	CMDRESULT=$?
	if [ $CMDRESULT -ne 0 ]
	then
		if [ "${DOWNLOADSERVER}" = "${FALLBACKSERVER}" ]
		then
			downloadError
		else
			# try using the fall-back server
			DOWNLOADSERVER=$FALLBACKSERVER
			SERVER=$FALLBACKSERVER
			wget --passive-ftp --tries=3 ${DOWNLOADSERVER}${FILETODOWNLOAD} > /dev/null 2>&1
			CMDRESULT=$?
			if [ $CMDRESULT -ne 0 ]
			then
				downloadError
			fi
		fi	
	fi
}

copyFile ()
{
	# Then we've got the file we wanted,
	# now we copy it to the f-prot directory.
	mv -f ${TEMPDIR}${FILETOCHECK} ${FPROTDIR}
	chmod 644 ${FPROTDIR}/${FILETOCHECK}
	CMDRESULT=$?

	if [ $CMDRESULT -ne 0 ]
	then
		copyError
	fi

	if [ $QUIET -eq 0 ]
	then
		fileInstallCompleted
	fi
}
newSignatureFile ()
{
	NEWFILE=${FILETOCHECK:0:4}
	if [ "$NEWFILE" = "SIGN" ]
	then
		NEWFILE="Application/Script viruses and Trojans" 
	elif [ "$NEWFILE" = "MACR" ]
	then
		NEWFILE="Document/Office/Macro viruses"
	else
		echo "Error: Unknown filetype"
		echo "Exiting..."
		exit -5
	fi

	echo "There's a new version of:"
	echo "\"${NEWFILE}\" signatures on the web."
	echo "Starting to download..."
}

# Here we start execution.

parseParameters $PARAMETERS
checkIfTempDirExists
changePermissionsOfTempDir
cleanTempDir
checkForExternalPrograms
downloadUpdateInfo 
getHTTPReturnValue
checkHTTPReturnValue
# we got a correct response

# Now we get and compare checksums of the
# files on the update server and the local 
# def files.
for i in `grep ^C ${TEMPDIR}${TMPFILE} | grep "DEF="`
do
	FILETOCHECK=${i#*:}
	FILETOCHECK=${FILETOCHECK%=*}
	REMOTEFILECHECKSUM=${i#*=}
	FILECHECKSUM=`"${FPROTDIR}"checksum "${DEFDIR}""${FILETOCHECK}" 0`
	CMDRESULT=$?
	if [ $CMDRESULT -ne 0 ]
	then
		if [ $CMDRESULT -eq 127 ]
		then
			echo "${FPROTDIR}checksum was not found. It should"
			echo "have been included in your f-prot package,"
			echo "please download it and install it on your"
			echo "computer, and try again."
			echo ""
			echo "Fatal error: exiting..."
			echo ""
			exit 127
		else
			echo "Unknown fatal error. Exiting..."
			exit -12
		fi
	fi		

	FILECHECKSUM=${FILECHECKSUM#*=}

	if [ $FILECHECKSUM != $REMOTEFILECHECKSUM ]
	then
		#download and install
		if [ $QUIET -eq 0 ]
		then
			if [ $UPDATED -eq 0 ]
			then
				printHeader
			fi
		fi

		UPDATED=1
		SERVER=`grep ^S ${TEMPDIR}${TMPFILE}`
		SERVER=${SERVER#*:}
		SERVER=${SERVER%/*}
		SERVER=${SERVER}/

		downloadFromServer			

		# Now we make sure that the files we downloaded are
		# the files we wanted!
		# (when downloading from mirror site's it can happen
		# that they haven't been able to update their site

		# before we start downloading, this is rare but it
		# can happen).
		FILECHECKSUM=`${FPROTDIR}checksum ${TEMPDIR}${FILETOCHECK} 0`
		FILECHECKSUM=${FILECHECKSUM#*=}
		if [ $REMOTEFILECHECKSUM = ${FILECHECKSUM} ]
		then
			copyFile
		else
			# The checksums do not match. This may be a mirror 
			# problem, so we will try once more with the 
			# fall-back server.
			if [ "${SERVER}" = "${FALLBACKSERVER}" ]
			then
				cleanTempDir
				checksumError
			fi
			# We don't need to include an else statement here
			# since we know that checksumError will exit.
			SERVER=$FALLBACKSERVER
			downloadFromServer

			FILECHECKSUM=`${FPROTDIR}checksum ${TEMPDIR}${FILETOCHECK} 0`
			FILECHECKSUM=${FILECHECKSUM#*=}

			if [ $REMOTEFILECHECKSUM = ${FILECHECKSUM} ]
			then
				copyFile
			else
				cleanTempDir
				checksumError
			fi
			
		fi
		
	
	else
		# If we get here it means our files are
		# up to date.
		echo -n ""
	fi
	
done

if [ $UPDATED -eq 1 ]
then
	if [ $QUIET -eq 0 ]
	then
		printUpdateCompleted	
	fi
else
	if [ $CRON -eq 0 ]
	then
		echo ""
		echo "Nothing to be done..."
		echo ""
	fi
fi

# Here we remove everything from the temporary directory.
cleanTempDir

exit 0
